@@ -1,4 +1,4 @@
-/* $Id: bogoreader.c,v 1.65 2005/03/27 22:54:33 m-a Exp $ */
+/* $Id: bogoreader.c,v 1.67 2006/05/29 14:28:00 relson Exp $ */
 
 /*****************************************************************************
 
@@ -28,7 +28,6 @@ AUTHORS: (C) Copyright 2003 by
 #include <ctype.h>
 #include <errno.h>
 #include <stdlib.h>
-#include <stdio.h>
 
 #include "bogoreader.h"
 #include "error.h"
@@ -359,7 +358,7 @@ static bool dir_next_mail(void)
     struct dirent *dirent;
     struct stat st;
 
-    for(;;) {
+    for (;;) {
 	if (reader_dir == NULL) {
 	    char *x = dir_name;
 	    /* open next directory */
