@@ -1,4 +1,4 @@
-/* $Id: datastore_db.c,v 1.218 2005/06/29 10:11:16 m-a Exp $ */
+/* $Id: datastore_db.c,v 1.224 2006/05/29 14:28:00 relson Exp $ */
 
 /*****************************************************************************
 
@@ -7,7 +7,7 @@ datastore_db.c -- implements the datastore, using Berkeley DB
 
 AUTHORS:
 Gyepi Sam <gyepi@praxis-sw.com>   2002 - 2003
-Matthias Andree <matthias.andree@gmx.de> 2003 - 2004
+Matthias Andree <matthias.andree@gmx.de> 2003 - 2005
 
 ******************************************************************************/
 
@@ -22,9 +22,6 @@ Matthias Andree <matthias.andree@gmx.de> 2003 - 2004
  *   deadlock
  * - document code changes
  * - conduct massive tests
- * - check if we really need the log files for "catastrophic recovery"
- *   or if we can remove them (see db_archive documentation)
- *   as the log files are *HUGE* even compared with the data base
  */
 
 /*
@@ -37,7 +34,6 @@ Matthias Andree <matthias.andree@gmx.de> 2003 - 2004
 #include <errno.h>
 #include <string.h>
 #include <stdlib.h>
-#include <unistd.h>		/* for SEEK_SET for SunOS 4.1.x */
 #include <sys/resource.h>
 #include <assert.h>
 
@@ -272,7 +268,7 @@ int db_lock(void *vhandle, int open_mode)
 	    e = errno = EAGAIN;
     } else {
 	/* have lock */
-	if (handle->fd > 0)
+	if (handle->fd >= 0)
 	    handle->locked = true;
     }
     return e;
@@ -396,6 +392,9 @@ void *dbe_init(bfpath *bfp)
     if (!check_path(bfp))
 	exit(EX_ERROR);
 
+    if (bfp->isdir)
+	return NULL;
+
     dsm_init(bfp);
     
     env = dsm->dsm_env_init(bfp);
@@ -458,13 +457,21 @@ static void check_db_version(void)
  * current resource limit and warn if file size is "close" (2 MB) to the
  * limit. errors from the system are ignored, no warning then.
  */
-static void check_fsize_limit(int fd, uint32_t pagesize) {
+static bool check_fsize_limit(int fd, uint32_t pagesize) {
+
+    bool ok = true;
+    static bool fPrinted = false;
+
 #ifndef __EMX__
     struct stat st;
     struct rlimit rl;
 
-    if (fstat(fd, &st)) return; /* ignore error */
-    if (getrlimit(RLIMIT_FSIZE, &rl)) return; /* ignore error */
+    // Only print this message once per run
+    if (fPrinted)
+	return ok;
+
+    if (fstat(fd, &st)) return ok; /* ignore error */
+    if (getrlimit(RLIMIT_FSIZE, &rl)) return ok; /* ignore error */
     if (rl.rlim_cur != (rlim_t)RLIM_INFINITY) {
 	/* WARNING: Be extremely careful that in these comparisons there
 	 * is no unsigned term, it will spoil everything as C will
@@ -473,18 +480,21 @@ static void check_fsize_limit(int fd, uint32_t pagesize) {
 	 * work when pagesize doesn't fit into signed long. ("requires"
 	 * 2**31 for file size and 32-bit integers to fail) */
 	if ((off_t)(rl.rlim_cur/pagesize) - st.st_size/(long)pagesize < 16) {
-	    print_error(__FILE__, __LINE__, "error: the data base file size is only 16 pages");
-	    print_error(__FILE__, __LINE__, "       below the resource limit. Cowardly refusing");
-	    print_error(__FILE__, __LINE__, "       to continue to avoid data base corruption.");
-	    exit(EX_ERROR);
+	    print_error(__FILE__, __LINE__, "error: the data base file size is within 16 pages");
+	    print_error(__FILE__, __LINE__, "       of the resource limit. Cowardly refusing");
+	    print_error(__FILE__, __LINE__, "       to write to avoid data base corruption.");
+	    ok = false;
 	}
+	else
 	if ((off_t)(rl.rlim_cur >> 20) - (st.st_size >> 20) < 2) {
+	    fPrinted = true;
 	    print_error(__FILE__, __LINE__, "warning: data base file size approaches resource limit.");
 	    print_error(__FILE__, __LINE__, "         write errors (bumping into the limit) can cause");
 	    print_error(__FILE__, __LINE__, "         data base corruption.");
 	}
     }
 #endif
+    return ok;
 }
 
 /* The old, pre-3.3 API will not fill in the page size with
@@ -702,7 +712,11 @@ retry_db_open:
 	    pagesize = 16384;
 
 	/* check file size limit */
-	check_fsize_limit(handle->fd, pagesize);
+	if (open_mode != DS_READ) {
+	    if (!check_fsize_limit(handle->fd, pagesize)) {
+		open_mode = DS_READ;
+	    }
+	}
 
 	/* Begin complex change ... */
 	e = dsm->dsm_lock(handle, open_mode);
